/*
 * Copyright (c) 2010-2012 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.vew.autoexport.VewLZZS
 *
 * de.inovat.vew.autoexport.VewLZZS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.vew.autoexport.VewLZZS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.vew.autoexport.VewLZZS; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.vew.autoexport;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Verwaltungsklasse fr alle LZZS, fr die der BASt-Export fr die Formate V20004, V2007 und AXL2009 durchgefhrt
 * werden soll.
 * <p/>
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kni (HCK)
 * @version $Revision: 780 $ / $Date: 2012-10-31 11:44:41 +0100 (Mi, 31 Okt 2012) $ / ($Author: HCK $)
 */
public class VewLZZS implements ClientSenderInterface {

    /** DebugLogger fr Debug-Ausgaben. */
    private static final Debug debug = Debug.getLogger();

    /** Datenverteilerverbindung */
    private static ClientDavInterface _dav;

    /** Die Konfiguration des DaV. */
    private static DataModel _konfiguration;

    //~ FELDER ================================================================

    private String     _ftpServer;
    private boolean    _isAenderungDatenstruktur;
    private boolean    _isBastBandUeberschreiben;
    List<SystemObject> _listeAxl;

    /** Die Liste der Systemobjekte, fr welche diese Verwaltungsklasse die Daten berechnet */
    List<SystemObject>   _listeLzzs;
    private SystemObject _verwaltungsObjektBastExportSteuerung;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Konstruktor der Verwaltungs Klasse.
     *
     * @param dav                            Die Datenverteilerverbindung
     * @param lzzs                           Das Systemobjekt, fr welches diese Verwaltungsklasse die Daten berechnen
     *                                       soll
     */

    /**
     * @param dav                      Die Datenverteilerverbindung
     * @param listeLzzs                Liste der LZZS-Systemobjekte, fr die BASt-Band Daten berechnet werden sollen
     * @param verwaltungsObjektBastExportSteuerung
     *                                 Verwaltungsobjekt, an dem die ATG zur Steuerung des BASt-Exports hngt.
     * @param ftpServer                Paramter fr den FTP-Server
     * @param isAenderungDatenstruktur Liegen eine nderung der Datenstruktur (BASt) vor?
     * @param isBastBandUeberschreiben Soll eventuell existierende BASt-Datei beim Export berschrieben werden?
     */
    public VewLZZS(ClientDavInterface dav, List<SystemObject> listeLzzs, SystemObject verwaltungsObjektBastExportSteuerung, String ftpServer, boolean isAenderungDatenstruktur, boolean isBastBandUeberschreiben) {
        _dav                                  = dav;
        _listeLzzs                            = listeLzzs;
        _verwaltungsObjektBastExportSteuerung = verwaltungsObjektBastExportSteuerung;
        _ftpServer                            = ftpServer;
        _isAenderungDatenstruktur             = isAenderungDatenstruktur;
        _isBastBandUeberschreiben             = isBastBandUeberschreiben;
        _konfiguration                        = _dav.getDataModel();
        _listeAxl                             = ermittleListeAxl(_listeLzzs);

        // Anmeldung auf das Versenden von Exportauftrgen
        DataDescription datenBeschreibung = new DataDescription(_konfiguration.getAttributeGroup("atg.bastExportSteuerung"), _konfiguration.getAspect("asp.anfrage"));

        try {
            _dav.subscribeSender(this, _verwaltungsObjektBastExportSteuerung, datenBeschreibung, SenderRole.sender());
        }
        catch (OneSubscriptionPerSendData e) {
            debug.error(String.format("Versuch der Doppelanmeldung der BASt-Export Datensteuerung [%s]%nProgramm wird beendet.", datenBeschreibung), e);
            System.exit(-1);
        }
    }

    //~ METHODEN ==============================================================

    /**
     * Methode, die den eigentlichen Auftrag ausfhrt.
     *
     * @param auftrag Der auszufhrende Auftrag.
     *
     * @return Meldung, die den Ablauf der Ausfhrung des Auftrags enthlt. Wird fr die Statusdarstellung verwendet.
     */
    public String ausfuehren(Auftrag auftrag) {
        List<SystemObject> listeAuftragsObjekte = new ArrayList<SystemObject>();

        switch (auftrag.getBastFormat()) {
        case V2004 :
        case V2007 :
            listeAuftragsObjekte = _listeLzzs;

            break;

        case AXL2009 :
            listeAuftragsObjekte = _listeAxl;

            break;
        }

        // Anmeldung auf das Versenden von Exportauftrgen
        DataDescription datenBeschreibung = new DataDescription(_konfiguration.getAttributeGroup("atg.bastExportSteuerung"), _konfiguration.getAspect("asp.anfrage"));

        // Datensatz fr die Statusinformationen
        Data              exportAuftragsDaten = _dav.createData(_konfiguration.getAttributeGroup("atg.bastExportSteuerung"));
        GregorianCalendar von                 = new GregorianCalendar();
        GregorianCalendar bis                 = new GregorianCalendar();

        von.setTimeInMillis(auftrag.getIntervallBeginn());
        bis.setTimeInMillis(auftrag.getIntervallEnde() - 1);
        exportAuftragsDaten.getUnscaledValue("vonTag").set(von.get(Calendar.DATE));
        exportAuftragsDaten.getUnscaledValue("vonMonat").set(von.get(Calendar.MONTH) + 1);
        exportAuftragsDaten.getUnscaledValue("vonJahr").set(von.get(Calendar.YEAR));
        exportAuftragsDaten.getUnscaledValue("bisTag").set(bis.get(Calendar.DATE));
        exportAuftragsDaten.getUnscaledValue("bisMonat").set(bis.get(Calendar.MONTH) + 1);
        exportAuftragsDaten.getUnscaledValue("bisJahr").set(bis.get(Calendar.YEAR));
        exportAuftragsDaten.getTextValue("BastBandAusgabeFormat").setText(auftrag.getBastFormat().toString());
        exportAuftragsDaten.getTextValue("BAStBandUeberschreiben").setText(_isBastBandUeberschreiben ? "Ja" : "Nein");
        exportAuftragsDaten.getTextValue("AenderungDatenstruktur").setText(_isAenderungDatenstruktur ? "Ja" : "Nein");
        exportAuftragsDaten.getTextValue("FTP-Server").setText(_ftpServer);
        exportAuftragsDaten.getReferenceArray("LangZeitZhlstellen").setLength(listeAuftragsObjekte.size());

        int i = 0;

        for (SystemObject lzzs : listeAuftragsObjekte) {
            exportAuftragsDaten.getReferenceArray("LangZeitZhlstellen").getReferenceValue(i).setSystemObject(lzzs);
            i++;
        }

        try {
            _dav.sendData(new ResultData(_verwaltungsObjektBastExportSteuerung, datenBeschreibung, System.currentTimeMillis(), exportAuftragsDaten));
        }
        catch (SendSubscriptionNotConfirmed e) {
            debug.error("Datenversand fr die BASt-Band-Steuerung fehlgeschlagen. Fehlende Anmeldung der Daten zum Versand.", e);
        }

        return "Auftrag ausgefhrt. " + auftrag;
    }

    /**
     * Sendesteuerung des Datenverteilers an die Applikation. Diese Methode muss von der Applikation implementiert
     * werden, um den Versand von Daten zu starten bzw. anzuhalten. Der Datenverteiler signalisiert damit einer Quelle
     * oder einem Sender dass mindestens ein Abnehmer bzw. kein Abnehmer mehr fr die zuvor angemeldeten Daten vorhanden
     * ist. Die Quelle wird damit aufgefordert den Versand von Daten zu starten bzw. zu stoppen.
     *
     * @param object          Das in der zugehrigen Sendeanmeldung angegebene Objekt, auf das sich die Sendesteuerung
     *                        bezieht.
     * @param dataDescription Beschreibende Informationen zu den angemeldeten Daten auf die sich die Sendesteuerung
     *                        bezieht.
     * @param state           Status der Sendesteuerung. Kann einen der Werte <code>START_SENDING</code>,
     *                        <code>STOP_SENDING</code>, <code>STOP_SENDING_NO_RIGHTS</code>,
     *                        <code>STOP_SENDING_NOT_A_VALID_SUBSCRIPTION</code> enthalten.
     *
     * @see #START_SENDING
     * @see #STOP_SENDING
     * @see #STOP_SENDING_NO_RIGHTS
     * @see #STOP_SENDING_NOT_A_VALID_SUBSCRIPTION
     */
    @Override
    public void dataRequest(SystemObject object, DataDescription dataDescription, byte state) {

        // Wird nicht untersttzt. Ergebnisdaten werden immer gesendet.
    }

    private List<SystemObject> ermittleListeAxl(List<SystemObject> listeLzzs) {
        List<SystemObject> listeAxl = new ArrayList<SystemObject>();

        // ToDo: ======================================================================================================
        // ToDo: ======================================================================================================
        // ToDo: Einfachlsung, einfach den ersten Eintrag nehmen, funktioniert in NI wegen der Sortierung bzgl. Namen
        // ToDo: ======================================================================================================
        // ToDo: ======================================================================================================
        listeAxl.add(listeLzzs.get(0));

        return listeAxl;
    }

    //~ GET METHODEN ==========================================================

    /**
     * Liefert die Pid des dieser Verwaltungseinheit zugeordnete LZZS Systemobjekt.
     *
     * @return Pid des dieser Verwaltungseinheit zugeordnete LZZS Systemobjekt.
     */
    public String getPid() {
        return "LISTE der LZZS";
    }

    /**
     * Diese Methode muss von der Applikation implementiert werden, um zu signalisieren, ob Sendesteuerungen erwnscht
     * sind und mit der Methode <code>dataRequest</code> verarbeitet werden. In der Implementierung dieser Methode
     * drfen keine synchronen Aufrufe, die auf Telegramme vom Datenverteiler warten (wie z.B. Konfigurationsanfragen)
     * durchgefhrt werden, da ansonsten ein Deadlock entsteht.
     *
     * @param object          Das in der zugehrigen Sendeanmeldung angegebene System-Objekt.
     * @param dataDescription Die in der zugehrigen Sendeanmeldung angegebenen beschreibenden Informationen der
     *                        angemeldeten Daten.
     *
     * @return <code>true</code>, falls Sendesteuerungen gewnscht sind, sonst <code>false</code>.
     *
     * @see #dataRequest
     */
    @Override
    public boolean isRequestSupported(SystemObject object, DataDescription dataDescription) {
        return false;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 31.10.12
